library ieee;
use ieee.std_logic_1164.all;

use work.controller_pkg.all;

-- Use this with a configuration like
-- configuration example of dev_board_addon is
--   for mapping
--     for controller_0 : controller
--       use entity work.my_controller(arch);
--     end for;
--   end for;
-- end configuration example;

entity dev_board_addon is
    port (
        -- Stuff for the controller
        clk : IN std_logic;
        reset_n : IN std_logic;

        led : OUT std_logic;

        awid : IN std_logic_vector(11 downto 0);
        awaddr : IN std_logic_vector(11 downto 0);
        awlen : IN std_logic_vector(3 downto 0);
        awsize : IN std_logic_vector(2 downto 0);
        awburst : IN std_logic_vector(1 downto 0);
        awprot : IN std_logic_vector(2 downto 0);
        awvalid : IN std_logic;
        awready : OUT std_logic;

        wid : IN std_logic_vector(11 downto 0);
        wdata : IN std_logic_vector(31 downto 0);
        wstrb : IN std_logic_vector(3 downto 0);
        wlast : IN std_logic;
        wvalid : IN std_logic;
        wready : OUT std_logic;

        bid : OUT std_logic_vector(11 downto 0);
        bresp : OUT std_logic_vector(1 downto 0);
        bvalid : OUT std_logic;
        bready : IN std_logic;

        arid : IN std_logic_vector(11 downto 0);
        araddr : IN std_logic_vector(11 downto 0);
        arlen : IN std_logic_vector(3 downto 0);
        arsize : IN std_logic_vector(2 downto 0);
        arburst : IN std_logic_vector(1 downto 0);
        arlock : IN std_logic_vector(1 downto 0);
        arcache : IN std_logic_vector(3 downto 0);
        arprot : IN std_logic_vector(2 downto 0);
        arvalid : IN std_logic;
        arready : OUT std_logic;

        rid : OUT std_logic_vector(11 downto 0);
        rdata : OUT std_logic_vector(31 downto 0);
        rresp : OUT std_logic_vector(1 downto 0);
        rlast : OUT std_logic;
        rvalid : OUT std_logic;
        rready : IN std_logic;

        -- Stuff for the dev board
        j101 : INOUT std_logic_vector(1 to 40);
        j102 : INOUT std_logic_vector(1 to 40)
    );
end entity dev_board_addon;

architecture mapping of dev_board_addon is
begin
    -- j101(1) - db(7)
    -- j101(2) - db(6)
    -- j101(3) - db(5)
    -- j101(4) - db(4)
    -- j101(5) - db(3)
    -- j101(6) - db(2)
    -- j101(7) - db(1)
    -- j101(8) - db(0)
    j101(10 to 18) <= (others => '0');
    -- j101(19) - dac_csn(0)
    -- j101(20) - dac_csn(1)
    -- j101(21) - dac_r_wn
    -- j101(22) - dac_ldac
    -- j101(23) - dac_rst
    -- j101(24) - dac_ls_dir
    -- j101(25) - dac_ls_oen(0)
    -- j101(26) - dac_ls_oen(1)
    j101(27 to 32) <= (others => '0');
    -- j101(33) - db(15)
    -- j101(34) - db(14)
    -- j101(35) - db(13)
    -- j101(36) - db(12)
    -- j101(37) - db(11)
    -- j101(38) - db(10)
    -- j101(39) - db(9)
    -- j101(40) - db(8)
    -- j102(1) - axis_amp_fault(0)
    -- j102(2) - axis_home(0)
    -- j102(3) - axis_forward_limit(0)
    -- j102(4) - axis_reverse_limit(0)
    -- j102(5) - axis_encoder_z(0)
    -- j102(6) - axis_encoder_b(0)
    -- j102(7) - axis_encoder_a(0)
    j102(8 to 25) <= (others => '0');
    -- j102(26) - axis_reverse_limit(1)
    j102(27) <= '0';
    -- j102(28) - axis_amp_fault(1)
    j102(29 to 31) <= (others => '0');
    -- j102(32) - axis_home(1)
    -- j102(33) - axis_inhibit_n(1)
    -- j102(34) - axis_forward_limit(1)
    j102(35) <= '0';
    -- j102(36) - axis_encoder_a(1)
    j102(37) <= '0';
    -- j102(38) - axis_encoder_b(1)
    -- j102(39) - axis_inhibit_n(0)
    -- j102(40) - axis_encoder_z(1)

    controller_0: controller_comp
        port map (
            clk => clk,
            reset_n => reset_n,

            led => led,

            awid => awid,
            awaddr => awaddr,
            awlen => awlen,
            awsize => awsize,
            awburst => awburst,
            awprot => awprot,
            awvalid => awvalid,
            awready => awready,

            wid => wid,
            wdata => wdata,
            wstrb => wstrb,
            wlast => wlast,
            wvalid => wvalid,
            wready => wready,

            bid => bid,
            bresp => bresp,
            bvalid => bvalid,
            bready => bready,

            arid => arid,
            araddr => araddr,
            arlen => arlen,
            arsize => arsize,
            arburst => arburst,
            arlock => arlock,
            arcache => arcache,
            arprot => arprot,
            arvalid => arvalid,
            arready => arready,

            rid => rid,
            rdata => rdata,
            rresp => rresp,
            rlast => rlast,
            rvalid => rvalid,
            rready => rready,

            dac_csn(1) => j101(20),
            dac_csn(0) => j101(19),
            dac_r_wn => j101(21),
            dac_ldac => j101(22),
            dac_rst => j101(23),

            db(0) => j101(8),
            db(1) => j101(7),
            db(2) => j101(6),
            db(3) => j101(5),
            db(4) => j101(4),
            db(5) => j101(3),
            db(6) => j101(2),
            db(7) => j101(1),
            db(8) => j101(40),
            db(9) => j101(39),
            db(10) => j101(38),
            db(11) => j101(37),
            db(12) => j101(36),
            db(13) => j101(35),
            db(14) => j101(34),
            db(15) => j101(33),

            dac_ls_dir => j101(24),
            dac_ls_oen(1) => j101(26),
            dac_ls_oen(0) => j101(25),

            axis_amp_fault(1) => j102(28),
            axis_amp_fault(0) => j102(1),
            axis_forward_limit(1) => j102(34),
            axis_forward_limit(0) => j102(3),
            axis_reverse_limit(1) => j102(26),
            axis_reverse_limit(0) => j102(4),
            axis_home(1) => j102(32),
            axis_home(0) => j102(2),
            axis_encoder_a(1) => j102(36),
            axis_encoder_a(0) => j102(7),
            axis_encoder_b(1) => j102(38),
            axis_encoder_b(0) => j102(6),
            axis_encoder_z(1) => j102(40),
            axis_encoder_z(0) => j102(5),
            axis_inhibit_n(1) => j102(33),
            axis_inhibit_n(0) => j102(39)
        );
end architecture mapping;
