library ieee;
use ieee.std_logic_1164.all;

package controller_pkg is
    component controller_comp is
        port (
            -- Control signals (not on board)
            clk : IN std_logic;
            reset_n : IN std_logic;

            led : OUT std_logic;

            awid : IN std_logic_vector(11 downto 0);
            awaddr : IN std_logic_vector(11 downto 0);
            awlen : IN std_logic_vector(3 downto 0);
            awsize : IN std_logic_vector(2 downto 0);
            awburst : IN std_logic_vector(1 downto 0);
            awprot : IN std_logic_vector(2 downto 0);
            awvalid : IN std_logic;
            awready : OUT std_logic;

            wid : IN std_logic_vector(11 downto 0);
            wdata : IN std_logic_vector(31 downto 0);
            wstrb : IN std_logic_vector(3 downto 0);
            wlast : IN std_logic;
            wvalid : IN std_logic;
            wready : OUT std_logic;

            bid : OUT std_logic_vector(11 downto 0);
            bresp : OUT std_logic_vector(1 downto 0);
            bvalid : OUT std_logic;
            bready : IN std_logic;

            arid : IN std_logic_vector(11 downto 0);
            araddr : IN std_logic_vector(11 downto 0);
            arlen : IN std_logic_vector(3 downto 0);
            arsize : IN std_logic_vector(2 downto 0);
            arburst : IN std_logic_vector(1 downto 0);
            arlock : IN std_logic_vector(1 downto 0);
            arcache : IN std_logic_vector(3 downto 0);
            arprot : IN std_logic_vector(2 downto 0);
            arvalid : IN std_logic;
            arready : OUT std_logic;

            rid : OUT std_logic_vector(11 downto 0);
            rdata : OUT std_logic_vector(31 downto 0);
            rresp : OUT std_logic_vector(1 downto 0);
            rlast : OUT std_logic;
            rvalid : OUT std_logic;
            rready : IN std_logic;

            -- dac controls
            dac_csn : OUT std_logic_vector(1 downto 0);
            dac_r_wn : OUT std_logic;
            dac_ldac : OUT std_logic;
            dac_rst : OUT std_logic;

            -- data lines to/from DAC (through a level shifter)
            db : INOUT std_logic_vector(15 downto 0);

            -- level shifter control
            dac_ls_dir : OUT std_logic; -- high - towards DAC
            dac_ls_oen : OUT std_logic_vector(1 downto 0); -- high - disable

            axis_amp_fault : IN std_logic_vector(1 downto 0);
            axis_forward_limit : IN std_logic_vector(1 downto 0);
            axis_reverse_limit : IN std_logic_vector(1 downto 0);
            axis_home : IN std_logic_vector(1 downto 0);
            axis_encoder_a : IN std_logic_vector(1 downto 0);
            axis_encoder_b : IN std_logic_vector(1 downto 0);
            axis_encoder_z : IN std_logic_vector(1 downto 0);
            axis_inhibit_n : OUT std_logic_vector(1 downto 0)
        );
    end component controller_comp;
end package controller_pkg;
