library ieee;
use ieee.std_logic_1164.all;

package dev_board_addon_pkg is
    component dev_board_addon_comp is
        port (
            -- Stuff for the controller
            clk : IN std_logic;
            reset_n : IN std_logic;

            led : OUT std_logic;

            awid : IN std_logic_vector(11 downto 0);
            awaddr : IN std_logic_vector(11 downto 0);
            awlen : IN std_logic_vector(3 downto 0);
            awsize : IN std_logic_vector(2 downto 0);
            awburst : IN std_logic_vector(1 downto 0);
            awprot : IN std_logic_vector(2 downto 0);
            awvalid : IN std_logic;
            awready : OUT std_logic;

            wid : IN std_logic_vector(11 downto 0);
            wdata : IN std_logic_vector(31 downto 0);
            wstrb : IN std_logic_vector(3 downto 0);
            wlast : IN std_logic;
            wvalid : IN std_logic;
            wready : OUT std_logic;

            bid : OUT std_logic_vector(11 downto 0);
            bresp : OUT std_logic_vector(1 downto 0);
            bvalid : OUT std_logic;
            bready : IN std_logic;

            arid : IN std_logic_vector(11 downto 0);
            araddr : IN std_logic_vector(11 downto 0);
            arlen : IN std_logic_vector(3 downto 0);
            arsize : IN std_logic_vector(2 downto 0);
            arburst : IN std_logic_vector(1 downto 0);
            arlock : IN std_logic_vector(1 downto 0);
            arcache : IN std_logic_vector(3 downto 0);
            arprot : IN std_logic_vector(2 downto 0);
            arvalid : IN std_logic;
            arready : OUT std_logic;

            rid : OUT std_logic_vector(11 downto 0);
            rdata : OUT std_logic_vector(31 downto 0);
            rresp : OUT std_logic_vector(1 downto 0);
            rlast : OUT std_logic;
            rvalid : OUT std_logic;
            rready : IN std_logic;

            -- Stuff on the devboard
            j101 : INOUT std_logic_vector(1 to 40);
            j102 : INOUT std_logic_vector(1 to 40)
        );
    end component dev_board_addon_comp;
end package dev_board_addon_pkg;

library ieee;
use ieee.std_logic_1164.all;

entity dev_board_mapping is
    port (
        clk : IN std_logic;
        reset_n : IN std_logic;

        led : OUT std_logic;

        awid : IN std_logic_vector(11 downto 0);
        awaddr : IN std_logic_vector(11 downto 0);
        awlen : IN std_logic_vector(3 downto 0);
        awsize : IN std_logic_vector(2 downto 0);
        awburst : IN std_logic_vector(1 downto 0);
        awprot : IN std_logic_vector(2 downto 0);
        awvalid : IN std_logic;
        awready : OUT std_logic;

        wid : IN std_logic_vector(11 downto 0);
        wdata : IN std_logic_vector(31 downto 0);
        wstrb : IN std_logic_vector(3 downto 0);
        wlast : IN std_logic;
        wvalid : IN std_logic;
        wready : OUT std_logic;

        bid : OUT std_logic_vector(11 downto 0);
        bresp : OUT std_logic_vector(1 downto 0);
        bvalid : OUT std_logic;
        bready : IN std_logic;

        arid : IN std_logic_vector(11 downto 0);
        araddr : IN std_logic_vector(11 downto 0);
        arlen : IN std_logic_vector(3 downto 0);
        arsize : IN std_logic_vector(2 downto 0);
        arburst : IN std_logic_vector(1 downto 0);
        arlock : IN std_logic_vector(1 downto 0);
        arcache : IN std_logic_vector(3 downto 0);
        arprot : IN std_logic_vector(2 downto 0);
        arvalid : IN std_logic;
        arready : OUT std_logic;

        rid : OUT std_logic_vector(11 downto 0);
        rdata : OUT std_logic_vector(31 downto 0);
        rresp : OUT std_logic_vector(1 downto 0);
        rlast : OUT std_logic;
        rvalid : OUT std_logic;
        rready : IN std_logic;

        gpio_0_d : INOUT std_logic_vector(35 downto 0);
        gpio_1_d : INOUT std_logic_vector(35 downto 0)
    );
end entity dev_board_mapping;

architecture mapping of dev_board_mapping is
    signal invalid : std_logic; -- This is VHDL being stupid
begin
    invalid <= 'X';

    dev_board: work.dev_board_addon_pkg.dev_board_addon_comp
        port map (
            clk => clk,
            reset_n => reset_n,

            led => led,

            awid => awid,
            awaddr => awaddr,
            awlen => awlen,
            awsize => awsize,
            awburst => awburst,
            awprot => awprot,
            awvalid => awvalid,
            awready => awready,

            wid => wid,
            wdata => wdata,
            wstrb => wstrb,
            wlast => wlast,
            wvalid => wvalid,
            wready => wready,

            bid => bid,
            bresp => bresp,
            bvalid => bvalid,
            bready => bready,

            arid => arid,
            araddr => araddr,
            arlen => arlen,
            arsize => arsize,
            arburst => arburst,
            arlock => arlock,
            arcache => arcache,
            arprot => arprot,
            arvalid => arvalid,
            arready => arready,

            rid => rid,
            rdata => rdata,
            rresp => rresp,
            rlast => rlast,
            rvalid => rvalid,
            rready => rready,

            j101(1) => gpio_0_d(0),
            j101(2) => gpio_0_d(1),
            j101(3) => gpio_0_d(2),
            j101(4) => gpio_0_d(3),
            j101(5) => gpio_0_d(4),
            j101(6) => gpio_0_d(5),
            j101(7) => gpio_0_d(6),
            j101(8) => gpio_0_d(7),
            j101(9) => gpio_0_d(8),
            j101(10) => gpio_0_d(9),
            j101(11) => invalid,
            j101(12) => invalid,
            j101(13) => gpio_0_d(10),
            j101(14) => gpio_0_d(11),
            j101(15) => gpio_0_d(12),
            j101(16) => gpio_0_d(13),
            j101(17) => gpio_0_d(14),
            j101(18) => gpio_0_d(15),
            j101(19) => gpio_0_d(16),
            j101(20) => gpio_0_d(17),
            j101(21) => gpio_0_d(18),
            j101(22) => gpio_0_d(19),
            j101(23) => gpio_0_d(20),
            j101(24) => gpio_0_d(21),
            j101(25) => gpio_0_d(22),
            j101(26) => gpio_0_d(23),
            j101(27) => gpio_0_d(24),
            j101(28) => gpio_0_d(25),
            j101(29) => invalid,
            j101(30) => invalid,
            j101(31) => gpio_0_d(26),
            j101(32) => gpio_0_d(27),
            j101(33) => gpio_0_d(28),
            j101(34) => gpio_0_d(29),
            j101(35) => gpio_0_d(30),
            j101(36) => gpio_0_d(31),
            j101(37) => gpio_0_d(32),
            j101(38) => gpio_0_d(33),
            j101(39) => gpio_0_d(34),
            j101(40) => gpio_0_d(35),
            j102(1) => gpio_1_d(0),
            j102(2) => gpio_1_d(1),
            j102(3) => gpio_1_d(2),
            j102(4) => gpio_1_d(3),
            j102(5) => gpio_1_d(4),
            j102(6) => gpio_1_d(5),
            j102(7) => gpio_1_d(6),
            j102(8) => gpio_1_d(7),
            j102(9) => gpio_1_d(8),
            j102(10) => gpio_1_d(9),
            j102(11) => invalid,
            j102(12) => invalid,
            j102(13) => gpio_1_d(10),
            j102(14) => gpio_1_d(11),
            j102(15) => gpio_1_d(12),
            j102(16) => gpio_1_d(13),
            j102(17) => gpio_1_d(14),
            j102(18) => gpio_1_d(15),
            j102(19) => gpio_1_d(16),
            j102(20) => gpio_1_d(17),
            j102(21) => gpio_1_d(18),
            j102(22) => gpio_1_d(19),
            j102(23) => gpio_1_d(20),
            j102(24) => gpio_1_d(21),
            j102(25) => gpio_1_d(22),
            j102(26) => gpio_1_d(23),
            j102(27) => gpio_1_d(24),
            j102(28) => gpio_1_d(25),
            j102(29) => invalid,
            j102(30) => invalid,
            j102(31) => gpio_1_d(26),
            j102(32) => gpio_1_d(27),
            j102(33) => gpio_1_d(28),
            j102(34) => gpio_1_d(29),
            j102(35) => gpio_1_d(30),
            j102(36) => gpio_1_d(31),
            j102(37) => gpio_1_d(32),
            j102(38) => gpio_1_d(33),
            j102(39) => gpio_1_d(34),
            j102(40) => gpio_1_d(35)
        );
end architecture mapping;
