library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


entity motion_controller_state_machine is
    port (
        clk : IN std_logic;
        reset : IN std_logic;

        position_command : IN unsigned(18 downto 0);
        command_direction : IN std_logic;

        position_actual : IN unsigned(18 downto 0);
        tick_tock : IN std_logic;

        home_strobe : IN std_logic;
        move_strobe : IN std_logic;
        stop_strobe : IN std_logic;

        decoder_homed : IN std_logic;

        negative_limit : IN std_logic;
        positive_limit : IN std_logic;

        dac_value : OUT unsigned(15 downto 0);
        decoder_home_strobe : OUT std_logic;

        done_strobe : OUT std_logic;
        interrupted_strobe : OUT std_logic
    );
end entity motion_controller_state_machine;


architecture state_machine of motion_controller_state_machine is
begin
    state_machine_proc:
    process(clk)
        type state_t is (reset_state, wait_home, home, idle, move);

        variable state : state_t := reset_state;

        variable command_direction_reg : std_logic;
        variable position_command_reg : unsigned(18 downto 0);

        variable target_tick_tock : std_logic;
    begin
        if clk'event and clk='1' then
            done_strobe <= '0';
            interrupted_strobe <= '0';
            dac_value <= "1000000000000000";
            decoder_home_strobe <= '0';

            if reset = '1' then
                state := reset_state;
            else
                -- TODO: Flag an error if move strobe in reset_state/home/move
                -- TODO: Flag an error if home strobe in home/move
                case state is
                    when reset_state =>
                        if home_strobe = '1' then
                            state := home;
                            command_direction_reg := command_direction;
                            decoder_home_strobe <= '1';
                        end if;
                    when wait_home =>
                        state := home;
                    when home =>
                        if stop_strobe = '1' then
                            state := reset_state;
                            interrupted_strobe <= '1';
                        end if;

                        if decoder_homed = '1' then
                            state := idle;
                            done_strobe <= '1';
                        else
                            if command_direction_reg = '1' then
                                dac_value <= "1100000000000000";
                            else
                                dac_value <= "0100000000000000";
                            end if;
                        end if;
                    when idle =>
                        if home_strobe = '1' then
                            -- Need to wait for quadrature to deassert homed
                            state := wait_home;

                            command_direction_reg := command_direction;

                            decoder_home_strobe <= '1';
                        elsif move_strobe = '1' then
                            if position_command /= position_actual then
                                state := move;

                                command_direction_reg := command_direction;
                                position_command_reg := position_command;

                                if command_direction = '1' then
                                    if position_command > position_actual then
                                        target_tick_tock := tick_tock;
                                    else
                                        target_tick_tock := not tick_tock;
                                    end if;
                                else
                                    if position_command < position_actual then
                                        target_tick_tock := tick_tock;
                                    else
                                        target_tick_tock := not tick_tock;
                                    end if;
                                end if;
                            end if;
                        end if;
                    when move =>
                        if stop_strobe = '1' then
                            state := idle;
                            interrupted_strobe <= '1';
                        end if;

                        if command_direction_reg = '1' then
                            dac_value <= "1100000000000000";
                        else
                            dac_value <= "0100000000000000";
                        end if;

                        if command_direction = '1' then
                            if position_actual >= position_command_reg and
                                    tick_tock = target_tick_tock then
                                --
                                dac_value <= "1000000000000000";
                                state := idle;
                                done_strobe <= '1';
                            end if;
                        else
                            if position_actual <= position_command_reg and
                                    tick_tock = target_tick_tock then
                                --
                                dac_value <= "1000000000000000";
                                state := idle;
                                done_strobe <= '1';
                            end if;
                        end if;
                end case;
            end if;
        end if;
    end process;
end architecture state_machine;
