library IEEE;
use IEEE.std_logic_1164.all;

entity simple_axi_slave_tb is
end entity simple_axi_slave_tb;

architecture tb of simple_axi_slave_tb is
    signal clk : std_logic;
    signal reset : std_logic;

    signal awid : std_logic_vector(11 downto 0);
    signal awaddr : std_logic_vector(11 downto 0);
    signal awlen : std_logic_vector(3 downto 0);
    signal awsize : std_logic_vector(2 downto 0);
    signal awburst : std_logic_vector(1 downto 0);
    signal awprot : std_logic_vector(2 downto 0);
    signal awvalid : std_logic;
    signal awready : std_logic;

    signal wid : std_logic_vector(11 downto 0);
    signal wdata : std_logic_vector(31 downto 0);
    signal wstrb : std_logic_vector(3 downto 0);
    signal wlast : std_logic;
    signal wvalid : std_logic;
    signal wready : std_logic;

    signal bid : std_logic_vector(11 downto 0);
    signal bresp : std_logic_vector(1 downto 0);
    signal bvalid : std_logic;
    signal bready : std_logic;

    signal arid : std_logic_vector(11 downto 0);
    signal araddr : std_logic_vector(11 downto 0);
    signal arlen : std_logic_vector(3 downto 0);
    signal arsize : std_logic_vector(2 downto 0);
    signal arburst : std_logic_vector(1 downto 0);
    signal arlock : std_logic_vector(1 downto 0);
    signal arcache : std_logic_vector(3 downto 0);
    signal arprot : std_logic_vector(2 downto 0);
    signal arvalid : std_logic;
    signal arready : std_logic;

    signal rid : std_logic_vector(11 downto 0);
    signal rdata : std_logic_vector(31 downto 0);
    signal rresp : std_logic_vector(1 downto 0);
    signal rlast : std_logic;
    signal rvalid : std_logic;
    signal rready : std_logic;

    signal reset_n : std_logic;


    signal artrans : std_logic;
begin
    reset_n <= not reset;

    dut: entity work.simple_axi_slave(simple)
        port map (
            clk => clk,
            reset_n => reset_n,

            awid => awid,
            awaddr => awaddr,
            awlen => awlen,
            awsize => awsize,
            awburst => awburst,
            awprot => awprot,
            awvalid => awvalid,
            awready => awready,

            wid => wid,
            wdata => wdata,
            wstrb => wstrb,
            wlast => wlast,
            wvalid => wvalid,
            wready => wready,

            bid => bid,
            bresp => bresp,
            bvalid => bvalid,
            bready => bready,

            arid => arid,
            araddr => araddr,
            arlen => arlen,
            arsize => arsize,
            arburst => arburst,
            arlock => arlock,
            arcache => arcache,
            arprot => arprot,
            arvalid => arvalid,
            arready => arready,

            rid => rid,
            rdata => rdata,
            rresp => rresp,
            rlast => rlast,
            rvalid => rvalid,
            rready => rready
        );

    tb_proc:
    process
    begin
        clk <= '0';
        reset <= '1';
        wait for 5 ns;

        clk <= '1';
        wait for 1 ns;
        reset <= '0';
        arvalid <= '1';
        arlen <= "0010";
        arsize <= "000";
        arburst <= "00";
        arlock <= "00";
        arcache <= "0000";
        arprot <= "000";
        rready <= '0';
        awaddr <= (2 => '1', others => '0');
        wdata <= (4 => '1', others => '0');
        wait for 5 ns;

        clk <= '0';
        wait for 5 ns;

        while arready = '0' loop
            clk <= '1';
            wait for 5 ns;

            clk <= '0';
            wait for 5 ns;
        end loop;

        clk <= '1';
        wait for 1 ns;
        arvalid <= '0';
        wait for 5 ns;

        clk <= '0';
        arvalid <= '0';
        wait for 5 ns;

        clk <= '1';
        wait for 5 ns;

        clk <= '0';
        wait for 5 ns;

        clk <= '1';
        wait for 5 ns;

        clk <= '0';
        rready <= '1';
        wait for 5 ns;

        for i in 0 to 20 loop
            clk <= '1';
            wait for 5 ns;

            clk <= '0';
            wait for 5 ns;
        end loop;

        clk <= '1';
        wait for 1 ns;
        reset <= '0';
        arvalid <= '1';
        arlen <= "0000";
        arsize <= "000";
        arburst <= "00";
        arlock <= "00";
        arcache <= "0000";
        arprot <= "000";
        rready <= '0';
        wait for 5 ns;

        clk <= '0';
        wait for 5 ns;

        while arready = '0' loop
            clk <= '1';
            wait for 5 ns;

            clk <= '0';
            wait for 5 ns;
        end loop;

        clk <= '1';
        wait for 1 ns;
        arvalid <= '0';
        wait for 5 ns;

        clk <= '0';
        wait for 5 ns;

        clk <= '1';
        wait for 5 ns;

        clk <= '0';
        wait for 5 ns;

        clk <= '1';
        wait for 5 ns;

        clk <= '0';
        rready <= '1';
        wait for 5 ns;

        for i in 0 to 20 loop
            clk <= '1';
            wait for 5 ns;

            clk <= '0';
            wait for 5 ns;
        end loop;

        clk <= '1';
        wait for 1 ns;
        reset <= '0';
        arvalid <= '1';
        arlen <= "0000";
        arsize <= "000";
        arburst <= "00";
        arlock <= "00";
        arcache <= "0000";
        arprot <= "000";
        rready <= '1';
        wait for 5 ns;

        clk <= '0';
        wait for 5 ns;

        while arready = '0' loop
            clk <= '1';
            wait for 5 ns;

            clk <= '0';
            wait for 5 ns;
        end loop;

        clk <= '1';
        wait for 1 ns;
        arvalid <= '0';
        wait for 5 ns;

        clk <= '0';
        wait for 5 ns;

        clk <= '1';
        wait for 5 ns;

        clk <= '0';
        wait for 5 ns;

        clk <= '1';
        wait for 5 ns;

        clk <= '0';
        rready <= '1';
        wait for 5 ns;

        for i in 0 to 20 loop
            clk <= '1';
            wait for 5 ns;

            clk <= '0';
            wait for 5 ns;
        end loop;


        clk <= '1';
        wait for 1 ns;
        reset <= '0';
        arvalid <= '0';
        arlen <= "0000";
        arsize <= "000";
        arburst <= "00";
        arlock <= "00";
        arcache <= "0000";
        arprot <= "000";
        rready <= '0';
        awvalid <= '1';
        wvalid <= '1';
        awaddr <= (2 => '1', others => '0');
        wdata <= (4 => '1', others => '0');
        wait for 5 ns;

        clk <= '0';
        wait for 5 ns;

        -- This works for this impl, but not in general.
        while awready = '0' or wready = '0' loop
            clk <= '1';
            wait for 1 ns;
            if awready = '1' then
                awvalid <= '0';
            end if;
            if wready = '1' then
                wvalid <= '0';
            end if;
            wait for 4 ns;

            clk <= '0';
            wait for 5 ns;
        end loop;

        clk <= '1';
        wait for 5 ns;

        clk <= '0';
        wait for 5 ns;

        clk <= '1';
        wait for 5 ns;

        clk <= '0';
        wait for 5 ns;

        clk <= '1';
        wait for 5 ns;

        clk <= '0';
        rready <= '1';
        wait for 5 ns;

        for i in 0 to 20 loop
            clk <= '1';
            wait for 5 ns;

            clk <= '0';
            wait for 5 ns;
        end loop;

        wait;
    end process;

    transaction_proc:
    process(clk)
    begin
        if clk'event and clk='1' then
            artrans <= arvalid and arready;
        end if;
    end process;
end architecture tb;
